******************************************************* 
***   PRODUCT: DIGITAL CONTROLLED POWER SUPPLY      *** 
***   WRITTEN BY: MARK WHEELER                      *** 
***   REV H      1/28/88                            *** 
***              8/24/93  ICE Version               *** 
******************************************************* 
*** 
***   RS232 COMMANDS
*** 
***   "V" REQUEST VOLTAGE
***   "I" REQUEST CURRENT
***   "S" SET VOLTAGE 
*** 
***   KEYPAD CONTROLS 
*** 
***   FN#0  READ SUPPLY VOLTAGE 
***   FN#1  READ SUPPLY CURRENT 
***   FN#3  READ EXTERNAL PROBE 
***   FN#4  TOGGLE SOUND
*** 
***   PORT CONFIGURATION: 
***   P NO.   DESCRIPTION              I/O 
*** 
***   P(10)...DATA LSB LATCH; A/D      I/O 
***   P(17)...DATA MSB LATCH; A/D      I/O 
***   P(36)...DISPLAY CLOCK             O
***   P(35)...DISPLAY DATA              O
***   P(34)...DISPLAY STROBE            O
***   P(33)...DISPLAY ENABLE            O
***   P(32)...LATCH 3  (DAC PORT1)      O
***   P(31)...LATCH 2  (KEYPAD IN)      O
***   P(30)...LATCH 1  (KEYPAD OUT)     O
***   P(40)...A/D SELECT  A0            O
***   P(41)...A/D SELECT  A1            O
***   P(42)...A/D SELECT  A2            O
***   P(43)...A/D OUTPUT ENABLE         O
***   P(44)...A/D START                 O
***   P(45)...A/D ADDR LATCH ENABLE     O
***   P(46)...A/D END OF CONVERSION     I
*** 
STRTROM   EQU  $C800    $F800 - ROM
STRTRAM   EQU  $A780    $80   - ROM
*
* 6321 CONSTANTS FOR ICE
*
PEREGA    EQU  $0             PERIPHERAL REG. A
DDRA      EQU  $0             DATA DIRECTION REG. A
CONTA     EQU  $1             CONTROL REG. A
PEREGB    EQU  $2             PERIPHERAL REG. B
DDRB      EQU  $2             DATA DIRECTION REG. B
CONTB     EQU  $3             CONTROL REG. B
A6321     EQU  $4000          BASE ADDRESS EMUL. 1&2
B6321     EQU  $6000          BASE ADDRESS EMUL. 3&4

PORT1     EQU  PEREGA+A6321   PORT 1 ADDRESS
PORT3     EQU  PEREGA+B6321   PORT 3 ADDRESS
PORT4     EQU  PEREGB+B6321   PORT 4 ADDRESS
*
* NORMAL CONSTANTS FOR 6801
*
P1DDR     EQU  00   DIRECTION PORT1 
P2DDR     EQU  01   DIRECTION PORT2 
P3DDR     EQU  04   DIRECTION PORT3 
P4DDR     EQU  05   DIRECTION PORT4 
* PORT1     EQU  02   PORT1 ADDR
* PORT3     EQU  06   PORT3 ADDR
* PORT4     EQU  07   PORT4 ADDR
RMCR      EQU  $10  
TRCSR     EQU  $11  
RDR       EQU  $12  
TDR       EQU  $13  
RCR       EQU  $14  
ENDRAM    EQU  $A7FF         $FF - ROM
CURRENT   EQU  $80  A/D CUR. ADDR 
VOLTAGE   EQU  $81  A/D VOLT ADDR 
PROBE_1   EQU  $82  
REF       EQU  $83  
PULSE_L   EQU  $04  VALUE FOR ZERO
PULSE_H   EQU  $24  VALUE FOR ONE 
CLK_HIGH  EQU  $44  CLK PULSE HIGH
CLK_LOW   EQU  $3F  CLOCK PULSE LOW 
STROBE    EQU  $14  STROBE SET
OUT_ENABLE EQU $04  OUTPUT ENABLE 
BEEPER    EQU  $84       
DA_SELECT EQU  $00      
* 
* BIT MAP FOR DISPLAY SEGMENT
*                     EDCGFAB A
DIG0      EQU  $77    *******     77 normal    
DIG1      EQU  $11   *       *
DIG2      EQU  $6B   *F      *B 
DIG3      EQU  $3B   *   G   *
DIG4      EQU  $1D    ******* 
DIG5      EQU  $3E   *       *
DIG6      EQU  $7E   *E      *C 
DIG7      EQU  $13   *   D   *
DIG8      EQU  $7F    ******* 
DIG9      EQU  $3F       
BLANK     EQU  $00  BLANK CHARACTER 
LET_E     EQU  $6E  LETTER E
LET_R     EQU  $48  LETTER R
LET_F     EQU  $4E  LETTER F
LET_N     EQU  $58  LETTER N
INC_CUR   EQU  $02
INC_VOL   EQU  $01
SCAN_H    EQU  $06  OUTPUT LATCH KEYPAD 
SCAN_L    EQU  $04  OUTPUT LATCH KEYPAD 
RD_KEY_H  EQU  $05  LATCH KEY VALUE 
RD_KEY_L  EQU  $04       
          ORG  STRTRAM
* 
DISP1     RMB  1    DIGIT #1
DISP2     RMB  1    DIGIT #2
DISP3     RMB  1    DIGIT #3
DISP4     RMB  1    DIGIT #4
DSP1      RMB  1    DIGIT #1
DSP2      RMB  1    DIGIT #2
DSP3      RMB  1    DIGIT #3
DSP4      RMB  1    DIGIT #4
SEEK1     RMB  1 
SEEK2     RMB  1 
SEEK3     RMB  1 
DPOINT    RMB  1    DECIMAL POINTS 1&2
ADVAL     RMB  1    DATA READ FROM A/D
ADAVG     RMB  2    AVERAGE A/D READING
ROW       RMB  1    KEY PAD ROW INPUT 
COL       RMB  1    KEY PAD COL INPUT 
MODE      RMB  1    CURRENT/VOLTAGE MODE
KEYVAL    RMB  1    0-9,#,* VALUE 
CDISP1    RMB  1 
CDISP2    RMB  1 
CDISP3    RMB  1 
VOUT      RMB  1    OUTPUT VOLTAGE
TONE      RMB  1 
SOUND     RMB  1 
DAVAL     RMB  1 
ERROR_CNT RMB  1 
STABLE_ERR RMB 1 
SCSTAT    RMB  1 
ENDTRA    RMB  1 
RECV_DATA RMB  1 
TX_DATA   RMB  1 
TRKFLG    RMB  1
TRACK     RMB  1  
REFRESH   RMB  1
*
          ORG  STRTROM 
INIT      
* ROM VERSION
*         LDS  #ENDRAM         SET STACK POINTER 
* 
*         RAM MODE CONTROL REGISTER SETUP 
* 
* CLOCK  PORT2     BIT3  BIT2    BAUD  BIT1  BIT0 
*  I       X         0    0      76800  0     0 
*  I       X         0    1      9600   0     1 
*  I      OUT        1    0      1200   1     0 
*  EXT     IN        1    1      300    1     1 
* 
          LDAA #$06            1200BAUD BIT2 DISABLED
          STAA RMCR
          LDAA #$0B            ENABLE THE TRANSMITTER & RECEIVER, BUT LEAVE
          STAA TRCSR           THE TRANSMIT & RECEIVE INTERRUPTS OFF 
*
*  ICE VERSION PORT INIT
*
          CLRB                SELECT DDRA PORT 1
          STAB CONTA+A6321
          CLRA                SET PORT 1 TO ALL INPUTS
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321

          STAB CONTA+B6321    SELECT DDRA PORT 3
          LDAA #$FF           SET PORT 3 TO ALL OUTPUTS
          STAA DDRA+B6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+B6321

          STAB CONTB+B6321    SELECT DDRA PORT 4
          LDAA #$BF           SET PORT 4 
          STAA DDRB+B6321
          LDAA #$04           SELECT CONTROLB
          STAA CONTB+B6321
*
*  MC6801 ROM VERSION PORT INIT
*
*         CLRA                 SET PORT 1
*         STAA P1DDR           I/O USED FOR A/D - D/A
          LDAA #$10            SET PORT 2
          STAA P2DDR           SAME FOR BOTH ICE / ROM
*         LDAA #$FF            SET PORT 3
*         STAA P3DDR 
*         LDAA #$BF            SET PORT 4
*         STAA P4DDR 

          CLR  DAVAL           SET OUTPUT VOLTAGE TO ZERO
          JSR  DAOUT           SEND VALUE TO D/A
          CLR  TONE
          CLR  MODE            SET MODE TO VOLTAGE 
          LDAA #OUT_ENABLE 
          STAA PORT3           CLEAR DISPLAY  
          LDAA #$80            SET OE'S
          STAA PORT4           
          LDAA #1
          STAA SOUND 
          CLR  TRKFLG
* 
* MAIN LINE PROGRAM BEGINS HERE.
* 
MAIN      JSR  RECV
          JSR  CHK_PAD         TEST KEYPAD STATUS
          BCC  NOKEY_YET                         
          JSR  KBDHNDLR
NOKEY_YET LDAA RECV_DATA 
          CMPA #73             TEST FOR CURRENT TO 232 PORT
          BNE  CHKPRB
          JMP  DCUR
CHKPRB    CMPA #80             TEST FOR PROBE TO 232 PORT
          BNE  CHKVOLT
          JMP  DPROB
CHKVOLT   CMPA #83             TEST FOR OUTPUT VOLTAGE
          BNE  CHKRFSH
          JMP  CH_VOLT
          RTS 
*
*
CHKRFSH   LDAA MODE            GET CURRENT MODE
          CMPA #0              TEST FOR VOLTAGE ONLY
          BNE  CHKCUR            
          JSR  READV           READ AND DISPLAY OUTPUT VOLTAGE
          LDAA TRKFLG
          BEQ  MAIN             
          LDAA TRACK
          CMPA ADVAL
          BEQ  MAIN
          JSR  SENDNEW
          JMP  MAIN
*
CHKCUR    CMPA #1              TEST FOR CURRENT ONLY
          BNE  TOGBOTH
DCUR      LDAA REFRESH
          DECA
          STAA REFRESH
          BNE  MAIN
          JSR  READC
          JMP  MAIN
*
TOGBOTH   CMPA #2              TEST FOR TOGGLE MODE
          BNE  CHKPROBE  
          JSR  READV           READ AND DISPLAY VOLTAGE
          JSR  DELAY
          JSR  READC           READ AND DISPLAY THE CURRENT
          JSR  DELAY
          JMP  MAIN
CHKPROBE  CMPA #3              TEST FOR EXTERNAL PROBE
          BNE  TOGAUD            
DPROB     JSR  GETPROBE 
          JMP  MAIN
TOGAUD    CMPA #4
          BNE  INVALID 
          LDAA SOUND 
          CMPA #1
          BNE  SOUNDON 
          CLR  SOUND 
          CLR  MODE
          JMP  MAIN
SOUNDON   LDAA #1
          STAA SOUND 
          CLR  MODE
          JMP  MAIN
INVALID   JSR  DIS_ERROR 
          JMP  MAIN
* 
* KEY BOARD HANDLER 
* 
KBDHNDLR  JSR  SCAN_KBD        SCAN THE KEY PAD
          JSR  TEST_KEY        TEST FOR VALID ROW COL
          STAA KEYVAL          STORE KEY VALUE 
          CMPA #$FF            TEST FOR INVALID KEY
          BEQ  CONT            NO KEY TO PROCESS 
          JSR  INTER           FIND USER RESPONCE
CONT      RTS
*
CH_VOLT   CLR  RECV_DATA 
          JSR  RECV
READ_3    LDAA RECV_DATA 
          BEQ  READ_3
          STAA TX_DATA 
          SUBA #$30
          STAA SEEK3 
          JSR  SEND
          CLR  RECV_DATA 
          JSR  RECV
TST_RD_3  LDAA RECV_DATA 
          BEQ  TST_RD_3
          CMPA #89 
          BNE  CH_VOLT
          CLR  RECV_DATA 
          JSR  RECV
READ_2    LDAA RECV_DATA 
          BEQ  READ_2
          STAA TX_DATA 
          SUBA #$30
          STAA SEEK2 
          JSR  SEND
          CLR  RECV_DATA 
          JSR  RECV
TST_RD_2  LDAA RECV_DATA 
          BEQ  TST_RD_2
          CMPA #89 
          BNE  CH_VOLT
          CLR  RECV_DATA 
          JSR  RECV
READ_1    LDAA RECV_DATA 
          BEQ  READ_1
          STAA TX_DATA 
          SUBA #$30
          STAA SEEK1 
          JSR  SEND
          CLR  RECV_DATA 
          JSR  RECV
TST_RD_1  LDAA RECV_DATA 
          BEQ  TST_RD_1
          CMPA #89 
          BNE  CH_VOLT
          JMP  SENDNEW 
* 
*  INTERPEAT KEY STROKE 
* 
INTER     LDAA KEYVAL
          CMPA #$0A
          BNE  C_RATE          CHANGE OUTPUT LEVEL 
          LDAA #LET_F
          STAA DISP4 
          LDAA #LET_N
          STAA DISP3 
          LDAA #BLANK
          STAA DISP2 
          STAA DISP1 
          LDAA #1
          STAA DPOINT
          JSR  UPDATE
          JSR  W_DIGIT 
          LDAA KEYVAL
          STAA MODE
          JSR  DEC_VFD 
          STAA DISP3 
          JSR  UPDATE
          JSR  DELAY 
          RTS 
*
*
*
C_RATE    LDAA MODE
          BEQ  PROCESS_V       TEST FOR TOG OR CUR 
          RTS 
*
*
*
PROCESS_V STAA DPOINT          CLEAR  DECIMAL
          LDAA #$0F            STORE A BLANK IN ALL
          STAA CDISP3          DISPLAYS
          STAA CDISP2
          STAA CDISP1
PRO_NEXT  LDAA KEYVAL
          CMPA #$0B            CHECK FOR DECIMAL PT. 
          BNE  DIGITS          PLACE 1ST DIGIT 
LAST      LDAA #02 
          STAA DPOINT
          JSR  UPDATE_REG      UPDATE DISPLAY
GET_100   JSR  W_DIGIT         LOOK FOR 100THS 
          LDAA KEYVAL          FETCH DIGIT 
          CMPA #$0A            TEST FOR 0-9
          BLT  GOT_100         GOT DIGIT 
          JSR  DIS_ERROR       WAS AN ERROR
WAITFKEY  JSR  SCAN_KBD        SCAN THE KEY PAD
          JSR  TEST_KEY        TEST FOR VALID ROW COL
          STAA KEYVAL          STORE KEY VALUE 
          CMPA #$FF            TEST FOR INVALID KEY
          BEQ  WAITFKEY        NO KEY TO PROCESS 
          BRA  INTER           TRY AGAIN 
GOT_100   STAA CDISP1          STORE DIGIT IN HUND.
          JSR  UPDATE_REG      UPDATE DISPLAY
          BRA  GOT_ALL         HAVE ALL CHARACTERS 
DIGITS    LDAA CDISP2          TEST FOR ONES PLACE 
          CMPA #$0F
          BNE  THIRD           WAS TENS PLACE
          LDAA KEYVAL          FETCH KEY VALUE 
          STAA CDISP2
          JSR  UPDATE_REG      UPDATE SCREEN 
GET_10    JSR  W_DIGIT         FETCH NEXT CHARACTER
          CMPA #$0A            TEST FOR WRONG KEY
          BNE  PRO_NEXT        PROCESS NEXT KEY
          JSR  DIS_ERROR       DISPLAY ERROR CODE
          JSR  UPDATE_REG      DISPLAY OLD DATA
          BRA  GET_10
THIRD     LDAA CDISP2          STORE 2ND IN 3RD DIGIT
          STAA CDISP3
          LDAA KEYVAL          UPDATE 2ND PLACE DIGIT
          STAA CDISP2
          BRA  LAST            FETCH TENTHS OF VOLT
* HAVE ALL KEYS ENTERED 
GOT_ALL   LDAA CDISP1
          STAA SEEK1 
          LDAA CDISP2
          CMPA #$0F
          BNE  SEEK2_OK
          CLRA 
SEEK2_OK  STAA SEEK2 
          LDAA CDISP3
          CMPA #$0F
          BNE  SEEK3_OK
          CLRA 
SEEK3_OK  STAA SEEK3 
*
* SEND NEW VOLTAGE SELETION TO D/A
* 
SENDNEW   JSR  DAOUT           REFRESH THE D/A OUTPUT
          JSR  READV           READ THE OUTPUT VOLTAGE AND DISPLAY
          LDAA DSP3 
          CMPA SEEK3 
          BEQ  SECSEG
          BGT  DECDA 
INCDA     LDAA DAVAL 
          INCA
          STAA DAVAL 
TST_MAX   CMPA #$FF  
          BNE  SENDNEW
OUTOFRANGE JSR DIS_ERROR 
          CLR  DAVAL 
          JSR  DAOUT 
          CLR  TRKFLG
          RTS 
DECDA     LDAA DAVAL 
          DECA
          STAA DAVAL 
          BRA  TST_MAX 
SECSEG    LDAA DSP2 
          CMPA SEEK2 
          BEQ  ONESEG
          BLT  INCDA 
          BRA  DECDA 
ONESEG    LDAA DSP1 
          CMPA SEEK1 
          BEQ  CHANGED
          BLT  INCDA 
          BEQ  CHANGED 
          BRA  DECDA 
CHANGED   LDAA ADVAL
          STAA TRACK
          LDAA #1
          STAA TRKFLG
          RTS 
* 
* WAIT FOR A VALID CHARACTER
* 
W_DIGIT   JSR  SCAN_KBD        SCAN THE KEY PAD
          JSR  TEST_KEY        DO ROW/COL COINVERSION
          CMPA #$FF            TEST FOR INVALID INPUT
          BEQ  W_DIGIT         WAS A BAD INPUT 
          STAA KEYVAL          SAVE KEY VALUE
          RTS                             RETURN
* 
*  UPDATE DISPLAY REGISTERS AND UPDATE SCREEN 
* 
UPDATE_REG     LDAA            CDISP1          FETCH HUND. PLACE 
          STAA DISP1           STORE IN 1ST DIGIT
          LDAA CDISP2          FETCH ONES PLACE
          STAA DISP2           STORE IN SECOND DIGIT 
          LDAA CDISP3          FETCH TENS PLACE
          STAA DISP3           STORE IN THIRD DIGIT
          LDAA #$0F            STORE A BLANK IN
          STAA DISP4           FOURTH DIGIT
          JSR  CONV_DISP       CONVERT TO BIT MAP
          JSR  UPDATE          UPDATE DISPLAY
          RTS                  RETURN
* 
* SCAN THE ENTIRE KEY PAD AND RETURN WITH KEY VALUE IN ACC. 
* 
SCAN_KBD  LDAB #$FE            GROUND 1ST ROW
          CLR  ROW             AND COLUME VALUES 
          CLR  COL 
SCAN_NXT  LDAA #$80            TRI STATE PORT1 
          STAA PORT4 
**************************
* ICE VERSION
          CLRA                SELECT DDRA PORT 1
          STAA CONTA+A6321
          LDAA #$FF           
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321
**************************
* ROM VERSION
*         LDAA #$FF            SET PORT1 TO OUTPUT 
*         STAA P1DDR 

          STAB PORT1           SEND SCAN VALUE 
          LDAA #SCAN_H         TOGGLE LATCH PIN
          STAA PORT3           FOR SCAN VALUE
          LDAA #SCAN_L 
          STAA PORT3 
**************************
* ICE VERSION
          CLRA                SELECT DDRA PORT 1
          STAA CONTA+A6321
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321
**************************
* ROM VERSION
*         CLRA
*         STAA P1DDR           RESET PORT1 TO INPUT


          LDAA #RD_KEY_H       LATCH IN DATA FROM
          STAA PORT3           KEY PAD 
          CLRA                 SET PORT OUTPUTS
          STAA PORT4 
          LDAA #RD_KEY_L 
          STAA PORT3           LATCH KEY ON BUSS 
          LDAA PORT1 
          CMPA #$FF  
          BNE  GOT_KEY         FOUND A VALID KEY 
          LSLB                 SCAN NEXT ROW 
          ORAB #01 
          CMPB #$DF  
          BEQ  NO_KEY
          JMP  SCAN_NXT        SCAN NEXT ROW 
GOT_KEY   STAA COL             SAVE COLUME 
          STAB ROW             SAVE ROW
RELEASE   LDAA #RD_KEY_H       LATCH IN DATA FROM
          ORAA #%10000000      ENABLE THE BEEPER  
          STAA PORT3           KEY PAD 
          LDAA #RD_KEY_L 
          ORAA #%10000000  
          STAA PORT3           LATCH KEY ON BUSS 
          LDAA PORT1 
          CMPA #$FF  
          BNE  RELEASE 
          LDAA #RD_KEY_L 
          ANDA #%01111111  
          STAA PORT3           LATCH KEY ON BUSS 
NO_KEY    LDAA #$80
          STAA PORT4           RESET OE'S HIGH 
          RTS 
* 
* CONVERT THE ROW;COL VALUES TO DECIMAL EQUIVILANT
* 
TEST_KEY  LDAA ROW 
          CMPA #$FD            TEST FOR 1,2,3
          BNE  T2              WAS NOT 
          LDAA COL 
          CMPA #$F7            TEST FOR A ONE
          BNE  TA2             TRY TWO 
          LDAA #1              WAS  A 1
          RTS 
TA2       CMPA #$FB            TEST FOR TWO
          BNE  TA3             TRY THREE 
          LDAA #2              WAS A 2 
          RTS 
TA3       CMPA #$FE            TEST FOR THREE
          BNE  ERROR           BAD DATA
          LDAA #3              WAS A 3 
          RTS 
T2        CMPA #$FB            TEST FOR 4,5,6
          BNE  T3              WAS NOT 
          LDAA COL 
          CMPA #$F7            TEST FOR A FOUR 
          BNE  TB2             TRY A FIVE
          LDAA #4              WAS A FOUR
          RTS 
TB2       CMPA #$FB            TEST FOR FIVE 
          BNE  TB3             TRY SIX 
          LDAA #5              WAS A FIVE
          RTS 
TB3       CMPA #$FD            TEST FOR SIX
          BNE  ERROR           WAS BAD DATA
          LDAA #6              WAS A SIX 
          RTS 
T3        CMPA #$F7            TEST FOR 7,8,9
          BNE  T4              TRY . AND # AND 0 
          LDAA COL 
          CMPA #$F7            TEST FOR A SEVEN
          BNE  TC2             TRY EIGHT 
          LDAA #7              WAS A SEVEN 
          RTS 
TC2       CMPA #$FB            TEST FOR EIGHT
          BNE  TC3             TRY NINE
          LDAA #8              WAS A EIGHT 
          RTS 
TC3       CMPA #$FD            TEST FOR NINE 
          BNE  ERROR           WAS AN ERROR
          LDAA #9              WAS A NINE
          RTS 
T4        CMPA #$EF            TEST FOR *,0
          BNE  T5
          LDAA COL 
          CMPA #$FB  
          BNE  TD2 
          LDAA #0              WAS A 0 
          RTS 
TD2       CMPA #$F7  
          BNE  ERROR 
          LDAA #$0B            WAS A * 
          RTS 
T5        CMPA #$FE            TEST FOR #
          BNE  ERROR 
          LDAA COL 
          CMPA #$EF  
          BNE  ERROR 
          LDAA #$0A            WAS A # 
          RTS 
ERROR     LDAA #$FF            ERROR 
          RTS 
*  
* READ CURRENT AND UPDATE DISPLAY 
* 
READC     JSR  READCURR        READ THE CURRENT
          STAA ADVAL 
          CMPA #$FF            TEST FOR OVER CURRENT 
          BNE  GOT_C           STILL OK
          JSR  DIS_ERROR       FLASH ERROR 
          BRA  READC           READ AGAIN
GOT_C     JSR  CONV_CUR
          BRA  UP_DISP
* 
* READ VOLTAGE AND UPDATE DISPLAY 
* 
READV     JSR  READREF         READ THE REFERENCE VOLTAGE        
          STAA ADVAL 
          JSR  READVOLT        READ THE OUTPUT VOLTAGE
          SUBA ADVAL           SUBTRACK REFERENCE 
          STAA ADVAL           STORE THE READING 
          CMPA #$FF            TEST FOR OVER VOLTAGE 
          BNE  GOT_V           WAS OK
          JSR  DIS_ERROR       DISPLAY ERROR 
          BRA  READV           READ AGAIN
GOT_V     JSR  CONV_VOL        CONVERT TO DECIMAL
          BRA  UP_DISP
* 
* READ THE EXTERNAL PROBE INPUT 
* 
GETPROBE  JSR  READ_P          READ THE PROBE VOLTAGE
          STAA ADVAL 
          CMPA #$FF            TEST FOR OVER VOLTAGE 
          BNE  GOT_PRB         WAS OK
          JSR  DIS_ERROR       DISPLAY ERROR 
          BRA  GETPROBE        READ AGAIN
GOT_PRB   JSR  CONV_VOL
UP_DISP   JSR  CONV_DISP 
          JSR  UPDATE
          RTS 
* 
* DISPLAY ERROR MESSAGE ON DISPLAY
* BLANK Err 
DIS_ERROR LDAA SOUND
          CMPA #1
          BNE  NOSOUND
          LDAA #BEEPER 
          STAA TONE
NOSOUND   LDAA DPOINT          SAVE CURRENT DECIMAL
          PSHA
          LDAA #BLANK          BLANK 1ST CHARACTER 
          STAA DISP4 
          STAA DPOINT          CLEAR DECIMALS
          LDAA #LET_E          FETCH LETTER "E"
          STAA DISP3           STORE IN 2ND PLACE
          LDAA #LET_R          FETCH LETTER "r"
          STAA DISP2           STORE IN 3RD PLACE
          STAA DISP1           STORE IN 4TH PLACE
          JSR  UPDATE          UPDATE DISPLAY
          JSR  DELAY           WAIT
          LDAA #BLANK          BLANK DISPLAY 
          STAA DISP4 
          STAA DISP3 
          STAA DISP2 
          STAA DISP1 
          CLR  TONE
          JSR  UPDATE          SEND BLANKS 
          JSR  DELAY           WAIT
          PULA                 RESTORE DECIMALS
          STAA DPOINT
          RTS 
* 
* DELAY 
* 
DELAY     PSHX                 STORE THE INDEX
          LDX  #$1FFF          SET DELAY TIME
W1        JSR  CHK_PAD         TEST KEYPAD STATUS
          BCS  EXITDLY         A KEY WAS PRESSED
          DEX                  DECREMENT THE DELAY COUNTER
          BNE  W1              NO KEY WAS PRESSED
EXITDLY   PULX
          RTS 

*
* TEST FOR A KEY
*
CHK_PAD   LDAA #$FF
          STAA KEYVAL
DEB_KEY   LDAA #$80            TRI STATE PORT1 
          STAA PORT4 
**************************
* ICE VERSION
          CLRA                SELECT DDRA PORT 1
          STAA CONTA+A6321
          LDAA #$FF           
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321
**************************
* ROM VERSION
*         LDAA #$FF            SET PORT1 TO OUTPUT 
*         STAA P1DDR 
          CLRA
          STAA PORT1           SEND SCAN VALUE 
          LDAA #SCAN_H         TOGGLE LATCH PIN
          ORAA TONE
          STAA PORT3           FOR SCAN VALUE
          LDAA #SCAN_L 
          ORAA TONE
          STAA PORT3 
**************************
* ICE VERSION
          CLRA                SELECT DDRA PORT 1
          STAA CONTA+A6321
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321
**************************
* ROM VERSION
*         CLRA
*         STAA P1DDR           RESET PORT1 TO INPUT
          LDAA #RD_KEY_H       LATCH IN DATA FROM
          ORAA TONE
          STAA PORT3           KEY PAD 
          CLRA                 SET PORT OUTPUTS
          STAA PORT4 
          LDAA #RD_KEY_L 
          ORAA TONE
          STAA PORT3           LATCH KEY ON BUSS 
          LDAA PORT1 
          LDAB #RD_KEY_L 
          ORAB TONE 
          STAB PORT3           LATCH KEY ON BUSS 
          LDAB #$80
          STAB PORT4           RESET OE'S HIGH 
          CMPA #$FF  
          BEQ  NOKEY_P
          CMPA KEYVAL
          BNE  DEB_PAD
          SEC                  SET CARRY FOR KEY PRESSED
          RTS
DEB_PAD   STAA KEYVAL
          CLRA
DEBOUNCE  DECA
          BNE  DEBOUNCE
          JMP  DEB_KEY
NOKEY_P   CLC                  CLEAR THE CARRY FLAG
          RTS
* 
* CONVERT ADVAL TO DECIMAL VALUE FOR A VOLTAGE SOURCE 
* 
CONV_VOL  LDAA #2              TURN ON DP2 
          STAA DPOINT
          LDAA #$0F            BLANK OUT LEADING 
          STAA DISP4           DIGIT 
          CLR  DISP1           DIGITS VALUES 
          CLR  DISP2 
          CLR  DISP3 
          LDAB ADVAL           FETCH THE A/D VALUE 
RD_NXT_VOL     CMPB            #00             TEST IT FOR COMPLEAT
          BEQ  XMIT_VOLT       DONE CONVERTING 
          LDAA DISP1           FETCH HUNDTH OF VOLT
          ADDA #INC_VOL        INCREMENT IT
          STAA DISP1 
          CMPA #$0A            TEST FOR > 9
          BNE  DEC             STILL < 9 
          LDAA #0              WAS > ; CLEAR HUNDTH
          STAA DISP1 
          LDAA DISP2           INCREMENT VOLTS 
          INCA
          STAA DISP2 
          CMPA #$0A            TEST VOLT COUNT > 9 
          BNE  DEC 
          LDAA #0              CLEAR VOLT COUNT
          STAA DISP2 
          LDAA DISP3           INCREMENT TEN VOLT
          INCA
          STAA DISP3 
DEC       DECB                            DECREMENT A/D COUNT 
          JMP  RD_NXT_VOL      NOT DONE YET
XMIT_VOLT LDAA RECV_DATA 
          CMPA #86 
          BEQ  SEND_232_DATA 
          CMPA #80 
          BNE  DONT_S
SEND_232_DATA  LDAA            DISP3 
          ADDA #$30
          STAA TX_DATA 
          JSR  SEND
          LDAA DISP2 
          ADDA #$30
          STAA TX_DATA 
          JSR  SEND
          LDAA #46 
          STAA TX_DATA 
          JSR  SEND
          LDAA DISP1 
          ADDA #$30
          STAA TX_DATA 
          JSR  SEND
          LDAA #$0D
          STAA TX_DATA 
          JSR  SEND
          LDAA #$0A
          STAA TX_DATA 
          JSR  SEND
          CLR  RECV_DATA 
DONT_S    RTS 
* 
* CONVERT ADVAL TO DECIMAL VALUE FOR CURRENT SOURCE 
* 
CONV_CUR  LDAA #1              TURN ON DP1 
          STAA DPOINT
          LDAA #$0F
          STAA DISP1           CLEAR ALL SEGMENTS
          LDAA #0
          STAA DISP2 
          STAA DISP3 
          STAA DISP4 
          LDAB ADVAL           FETCH A/D VALUE 
RD_NXT    CMPB #00 
          BEQ  XMIT_CUR 
          LDAA DISP2 
          ADDA #INC_CUR
          STAA DISP2 
          CMPA #$0A
          BNE  DEC_NXT 
          LDAA #0
          STAA DISP2 
          LDAA DISP3 
          INCA
          STAA DISP3 
          CMPA #$0A
          BNE  DEC_NXT 
          LDAA #0
          STAA DISP3 
          LDAA DISP4 
          INCA
          STAA DISP4 
DEC_NXT   DECB
          JMP  RD_NXT
XMIT_CUR  LDAA RECV_DATA 
          CMPA #73 
          BNE  DONT_SC 
          LDAA DISP4 
          ADDA #$30
          STAA TX_DATA 
          JSR  SEND
          LDAA #46 
          STAA TX_DATA 
          JSR  SEND
          LDAA DISP3 
          ADDA #$30
          STAA TX_DATA 
          JSR  SEND
          LDAA DISP2 
          ADDA #$30
          STAA TX_DATA 
          JSR  SEND
          LDAA #$0D
          STAA TX_DATA 
          JSR  SEND
          LDAA #$0A
          STAA TX_DATA 
          JSR  SEND
          CLR  RECV_DATA 
DONT_SC   RTS 
* 
* PROCESS DECIMAL VALUES FOR VFD DISPLAY
* 
CONV_DISP LDAA DISP4           FETCH 4TH DISPLAY 
          STAA DSP4
          JSR  DEC_VFD         CONVERT IT
          STAA DISP4           RESTORE VALUE 
          LDAA DISP3           FETCH 3RD DISPLAY 
          STAA DSP3
          JSR  DEC_VFD         CONVERT IT
          STAA DISP3           RESTORE VALUE 
          LDAA DISP2           FETCH 2ND DISPLAY 
          STAA DSP2
          JSR  DEC_VFD         CONVERT IT
          STAA DISP2           RESTORE VALUE 
          LDAA DISP1           FETCH 1ST DISPLAY 
          STAA DSP1
          JSR  DEC_VFD         CONVERT IT
          STAA DISP1           RESTORE VALUE 
          RTS                  RETURN
*
* TEST DECIMAL VALUE AND REPLACE WITH BIT MAP VALUE 
*
DEC_VFD   CMPA #9              TEST DIGIT FOR 9
          BNE  C_08            NO: TEST NXT VALUE
          LDAA #DIG9           FETCH CHARACTER 9 
          RTS                             RETURN
C_08      CMPA #8
          BNE  C_07
          LDAA #DIG8 
          RTS 
C_07      CMPA #7
          BNE  C_06
          LDAA #DIG7 
          RTS 
C_06      CMPA #6
          BNE  C_05
          LDAA #DIG6 
          RTS 
C_05      CMPA #5
          BNE  C_04
          LDAA #DIG5 
          RTS 
C_04      CMPA #4
          BNE  C_03
          LDAA #DIG4 
          RTS 
C_03      CMPA #3
          BNE  C_02
          LDAA #DIG3 
          RTS 
C_02      CMPA #2
          BNE  C_01
          LDAA #DIG2 
          RTS 
C_01      CMPA #1
          BNE  C_00
          LDAA #DIG1 
          RTS 
C_00      CMPA #0
          BNE  ERR 
          LDAA #DIG0 
          RTS 
ERR       LDAA #BLANK
          RTS 
* 
* UPDATE SCREEN 
* SEND DP1;DSP1;DPS2;DSP3;DSP4;DP2 TO DISPLAY 
UPDATE    LDAA DPOINT
          RORA                            TEST FOR DP1  ON
          BCS  SET_DP1         TURN ON DECIMAL 1 
          LDAA #PULSE_L        DECIMAL POINT1 OFF
          BRA  SEND_DP1
SET_DP1   LDAA #PULSE_H        SET HIGH
SEND_DP1  STAA PORT3           SEND DATA TO PORT 
          ORAA #CLK_HIGH       SET CLOCK PULSE 
          STAA PORT3 
          ANDA #CLK_LOW        CLEAR THE CLOCK 
          STAA PORT3 
          LDAA DISP1          SEND 1ST DIGIT 
          JSR  SERIAL_OUT
          LDAA DISP2          SEND 2ND DIGIT 
          JSR  SERIAL_OUT
          LDAA DISP3          SEND 3RD DIGIT 
          JSR  SERIAL_OUT
          LDAA DISP4          SEND 4TH DIGIT 
          JSR  SERIAL_OUT
          LDAA DPOINT
          RORA
          RORA
          BCS  SET_DP2 
          LDAA #PULSE_L
          BRA  SEND_DP2
SET_DP2   LDAA #PULSE_H
SEND_DP2  STAA PORT3 
          ORAA #CLK_HIGH 
          STAA PORT3 
          ANDA #CLK_LOW
          STAA PORT3 
          LDAA #STROBE 
          STAA PORT3 
          LDAA #OUT_ENABLE 
          STAA PORT3 
          RTS 
* 
* DUMP SEGMENT CODE TO DISPLAY
* 
SERIAL_OUT LDAB #7             SET COUNT FOR SEVEN 
NXT_BIT   RORA                 ROTATE DATA 
          PSHA                 SAVE DATA ON STACK
          BCS  BIT_HIGH        TEST FOR ONE / ZERO 
          LDAA #PULSE_L        WAS A ZERO PULSE
          BRA  DUMP_IT         SEND PULSE TO DISP
BIT_HIGH  LDAA #PULSE_H        WAS A HIGH
DUMP_IT   STAA PORT3 
          ORAA #CLK_HIGH       SET CLOCK PIN HIGH
          ORAA #OUT_ENABLE 
          STAA PORT3 
          ANDA #CLK_LOW        RESET CLOCK PULSE 
          STAA PORT3 
          PULA                            RECOVER DATA
          DECB                            DECREMENT THE CLOCK 
          BNE  NXT_BIT 
          RTS 
* 
* READ SUPPLY CURRENT  RETURN VALUE IN ACCUM. 
* 
READCURR  LDAA #CURRENT        READ A/D CHANNEL 0
          BRA  READIT
* 
* READ EXTERNAL PROBE 
* 
READ_P    LDAA #PROBE_1
          BRA  READIT
* 
* READ SUPPLY VOLTAGE  RETURN VALUE IN ACCUM. 
* 
READREF   LDAA #REF
          BRA  READIT
READVOLT  LDAA #VOLTAGE        READ A/D CHANNEL 1
READIT    STAA PORT4           SET ADDR PORT 
          NOP
          NOP
          ORAA #$20            LATCH THE ADDRESS               
          STAA PORT4 
          STAA PORT4 
          ORAA #$10            START THE CONVERSION
          STAA PORT4 
          ANDA #$8F            CLEAR THE START AND ALE
          STAA PORT4 
          ORAA #$88            SET OE PIN HIGH 
          STAA PORT4           ENABLE A/D OUTPUTS
          PSHX   
          LDX  #20
WAIT_STRT DEX
          BNE  WAIT_STRT
          PULX
WAIT_EOC  LDAA PORT4 
          ANDA #$40            MASK OFF EOC BIT
          BEQ  WAIT_EOC        CONVERSION SIGNAL 
          LDAA PORT1           READ DATA FROM A/D
          RTS 
*
* SEND DATA TO D/A CONVERTER
*
DAOUT     LDAA #$80
          STAA PORT4 
**************************
* ICE VERSION
          CLRA                SELECT DDRA PORT 1
          STAA CONTA+A6321
          LDAA #$FF           
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321
**************************
* ROM VERSION
*         LDAA #$FF  
*         STAA P1DDR 
          LDAA DAVAL 
          STAA PORT1 
          LDAA #DA_SELECT
          STAA PORT3 
          LDAA #OUT_ENABLE 
          STAA PORT3 
**************************
* ICE VERSION
          CLRA                SELECT DDRA PORT 1
          STAA CONTA+A6321
          STAA DDRA+A6321
          LDAA #$04           SELECT CONTROLA
          STAA CONTA+A6321
**************************
* ROM VERSION
*         LDAA #0
*         STAA P1DDR 
          RTS 
* 
*  READ BUFFER FROM SERIAL PORT 
* 
RECV     
          LDAA TRCSR 
          ORAA #$18       
          STAA TRCSR 
          CLI 
          RTS 
  
* 
*  SEND BUFFER TO SERIAL PORT 
* 
SEND      
          SEI 
          LDAA TX_DATA         SEND ASCII A
          STAA TDR 
          CLR  ENDTRA
          LDAA #$04        
          ORAA TRCSR 
          STAA TRCSR 
          CLI 
LODM1     LDAA ENDTRA
          BEQ  LODM1 
          SEI 
          RTS 
*
* SERVICE ROUTINE FOR IRQ2
*
IRQ2      LDAA TRCSR              *XMIT/REC CONTROL STATUS REG 
          TAB                                *MAKE A COPY 
          ORAB #$DB               *OR WITH DB (11011011) 
          CMPB #$FF               *CHECK IF BITS 5H,1 ARE SET TDRE,TIE 
          BEQ  TRANSM             *IF SO GO TO TRANSM
          TAB                                *RESTORE THE TRCSR VALUE 
          ORAB #$B7                *OR WITH B7 (10110111)
          CMPB #$FF               *CHECK IF BITS 6H,3 ARE SET ORFE,RE
          BNE  NOER4              *IF SO ERROR 
          JMP  ERROR_232 
NOER4     TAB                     *ELSE RESTORE THE TRCSR
          ORAB #$77               *OR WITH 77 (01110111) 
          CMPB #$FF               *CHECK IF BITS 7H,3 ARE SET   RDRF,RE
          BNE  LEAVE              *IF SO RECEIVE 
          LDAA RDR 
          STAA RECV_DATA 
          SEI 
          RTI 
TRANSM    LDAA TRCSR 
          LDAB RDR          *DUMMY READ TO CLEAR ANY PENDING RECEIVE INTERRUPTS
          ORAA #$18         *OR WITH (00011000) TO SET RIE, RE & NOT WAKEUP
          ANDA #$FA         *AND WITH (11111010) TO CLEAR TIE & WAKEUP 
          STAA TRCSR 
          LDAA #1
          STAA ENDTRA       *ENDTRA <- 1 
LEAVE     RTI 
ERROR_232 CLR  TRCSR        * DISABLE TRANSMIT AND RECEIVE 
CHKSM     RTI 
* 
*  INVALID INTERRUPT REQUIESTED 
* 
INVALID_IRQ  RTI 
* 
          ORG  $CFF0           FFF0 - ROM
          FDB  IRQ2            SCI 
          FDB  INVALID_IRQ     TIMER OVERFLOW
          FDB  INVALID_IRQ     OUTPUT COMPARE
          FDB  INVALID_IRQ     INPUT CAPTURE 
          FDB  INVALID_IRQ     IRQ1
          FDB  INVALID_IRQ     SWI 
          FDB  INVALID_IRQ     NMI 
          FDB  INIT            RESET 
          END 

